<?php

namespace TurboSMSWoocommerceViberEdition;

/**
 * Class Settings
 * @package TurboSMSWoocommerceViberEdition
 */
class Settings extends ASettings {

	public $prefix = 'twve';

	public $statuses = array();

	/**
	 * Run
	 */
	function run() {
		parent::run();

		$this->statuses = array(
			'pending'    => __( 'Pending', 'woocommerce' ),
			'failed'     => __( 'Failed', 'woocommerce' ),
			'on-hold'    => _x( 'On hold', 'Order status', 'woocommerce' ),
			'processing' => __( 'Processing', 'woocommerce' ),
			'completed'  => __( 'Completed', 'woocommerce' ),
			'refunded'   => __( 'Refunded', 'woocommerce' ),
			'cancelled'  => __( 'Cancelled', 'woocommerce' ),
		);

		add_action( 'admin_menu', array( $this, 'plugin_menu' ) );

		register_activation_hook( __FILE__, array( $this, 'activate' ) );

		add_action( 'init', array( $this, 'woocommerce_init' ), 1000 );

	}

	function woocommerce_init() {
		if ( ! class_exists( 'WooCommerce' ) ) {
			return;
		}
		$wc_statuses = \wc_get_order_statuses();

		foreach ( $wc_statuses as $key => $title ) {
			$hook = str_replace( 'wc-', '', $key );
			if ( ! isset( $this->statuses[ $hook ] ) ) {
				$this->statuses[ $hook ] = $title;
			}
			add_action( "woocommerce_order_status_{$hook}",
				array( $this, 'on_status_changed' )
			);
		}

		add_action( 'woocommerce_new_order', [ $this, 'on_new_order' ] );
	}

	function do_send( $order_id, $key ) {
		$order         = wc_get_order( $order_id );
		$status        = $order->get_status();
		$send_to_user  = $this->get_setting( "send_to_user_on_$key" ) === 'yes';
		$send_to_admin = $this->get_setting( "send_to_admin_on_$key" ) === 'yes';

		if ( ! $send_to_user && ! $send_to_admin ) {
			return;
		}

		$user_mes = $this->get_setting( "user_$key" );
		$user_mes = apply_filters( 'twve_user_message', $user_mes, $order, $order_id, $status );

		$admin_mes = $this->get_setting( "admin_$key" );
		$admin_mes = apply_filters( 'twve_admin_message', $admin_mes, $order, $order_id, $status );

		# Turbosms credentials
		$send_method = $this->get_setting( 'send_method' );

		$sms_sender   = $this->get_setting( 'sms_sender' );
		$viber_sender = $this->get_setting( 'viber_sender' );
		$token        = $this->get_setting( 'http_api_token' );

		$sms = Api::get_instance()->init( $sms_sender, $viber_sender, $token );

		$admin_phones = $this->get_setting( 'phone' );
		$admin_phones = explode( ',', $admin_phones );
		$admin_phones = array_map( 'trim', $admin_phones );

		if ( $send_to_admin && $admin_phones ) {
			foreach ( $admin_phones as $admin_phone ) {
				$admin_phone = Plugin::sanitize_phone( $admin_phone );
				$sms->sendMessage( $admin_mes, $admin_phone, $send_method );
			}
		}

		$user_phone = $order->get_billing_phone();
		if ( $send_to_user && $user_phone ) {
			$user_phone = Plugin::sanitize_phone( $user_phone );
			$sms->sendMessage( $user_mes, $user_phone, $send_method );
		}
	}

	/**
	 * Отправка СМС когда создан новый заказ
	 *
	 * @param $order_id
	 */
	function on_new_order( $order_id ) {
		$key = 'new_order';
		$this->do_send( $order_id, $key );
	}

	/**
	 * Отправка СМС когда изменен статус
	 *
	 * @param $order_id
	 */
	function on_status_changed( $order_id ) {
		$order  = wc_get_order( $order_id );
		$status = $order->get_status();
		$key    = $status;
		$this->do_send( $order_id, $key );
	}

	/**
	 * Меню плагина
	 */
	function plugin_menu() {
		add_plugins_page(
			__( 'TurboSMS settings', 'turbosms-woocommerce-viber-edition' ),
			'TurboSMS',
			'manage_options',
			'turbosms-woocommerce-viber-edition',
			array( $this, 'render' )
		);
	}

	/**
	 * Хук активации
	 */
	function activate() {
		$this->add_setting( 'sender', '' );
		$this->add_setting( 'login', '' );
		$this->add_setting( 'password', '' );
		$this->add_setting( 'phone', '' );

		foreach ( $this->statuses as $key => $label ) {
			$this->add_setting( "admin_$key", '' );
			$this->add_setting( "send_to_admin_on_$key", '' );
			$this->add_setting( "user_$key", '' );
			$this->add_setting( "send_to_user_on_$key", '' );
		}
	}

	/**
	 * Страница настроек
	 */
	function render() {
		include_once __DIR__ . '/inc/admin-settings.php';
	}

	private static $instance = null;

	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	private function __clone() {
	}

	private function __construct() {
	}
}
