<?php


namespace TurboSMSWoocommerceViberEdition;


class Api
{
	/**
	 * @var string
	 */
	protected $base = 'https://api.turbosms.ua';
	/**
	 * @var string
	 */
	protected $token;
	/**
	 * @var string
	 */
	protected $smsSender;
	/**
	 * @var string
	 */
	protected $viberSender;
	/**
	 * @var array
	 */
	protected $args;
	/**
	 * @var string
	 */
	protected $message = '';
	/**
	 * @var array
	 */
	protected $data = array();
	/**
	 * @var string
	 */
	protected $response_status;

	/**
	 * Api constructor.
	 *
	 * @param $smsSender string
	 * @param $viberSender string
	 * @param $token string
	 *
	 * @return Api
	 */
	function init($smsSender, $viberSender, $token)
	{
		$this->smsSender   = $smsSender;
		$this->viberSender = $viberSender;
		$this->token       = $token;
		$this->args        = array(
			'sslverify' => false,
			'timeout'   => 15,
			'headers'   => array(
				'Content-Type'  => 'application/x-www-form-urlencoded',
				'Authorization' => 'Basic ' . $this->token
			),
		);

		return $this;
	}

	/**
	 * @return array|string|null
	 */
	public function getBalance()
	{
		$this->call('user', 'balance');
		if (!$this->hasData('response_result.balance')) {
			return $this->message;
		}

		return $this->getData('response_result.balance');
	}

	/**
	 * @param $message string
	 * @param $to string
	 * @param $method string (sms|viber|hybrid)
	 *
	 * @return array|bool|string
	 */
	public function sendMessage($message, $to, $method = 'sms')
	{
		$data = array(
			'recipients' => [$to]
		);

		if ($method === 'sms') {
			$data['sms'] = [
				'sender' => $this->smsSender,
				'text'   => $message
			];
		} elseif ($method === 'viber') {
			$data['viber'] = [
				'sender' => $this->viberSender,
				'text'   => $message
			];
		} else {
			$data['viber'] = [
				'sender' => $this->viberSender,
				'text'   => $message
			];
			$data['sms']   = [
				'sender' => $this->smsSender,
				'text'   => $message
			];
		}

		$this->call('message', 'send', $data);
		if (!$this->hasData('response_result')) {
			return $this->message;
		}

		return $this->getData('response_result');
	}

	/**
	 * @return bool
	 */
	public function isSuccess()
	{
		return $this->response_status === 'OK';
	}

	/**
	 * @param $module string
	 * @param $method string
	 * @param $data array
	 *
	 * @return self
	 */
	protected function call($module, $method, $data = [])
	{
		$url = "{$this->base}/{$module}/{$method}.json";

		$this->args['body'] = $data;

		$response = wp_remote_post($url, $this->args);

		if (\is_wp_error($response)) {
			$this->message = $response->get_error_message();

			return $this;
		}

		$json = json_decode($response['body']);

		$this->data    = $json;
		$this->message = $json->response_status;

		return $this;
	}

	/**
	 * @param  null|string  $subParam
	 *
	 * @return array|null
	 */
	protected function getData($subParam = null)
	{
		if (!empty($subParam)) {
			if (!is_object($this->data)) {
				return null;
			}

			$subQ = explode('.', $subParam);
			$data = $this->data;
			foreach ($subQ as $subKey) {
				if (is_object($data) && property_exists($data, $subKey)) {
					$data = $data->{$subKey};
				} elseif (is_array($data) && array_key_exists($subKey, $data)) {
					$data = $data[$subKey];
				} else {
					return null;
				}
			}

			return $data;
		}

		return !empty($this->data) ? $this->data : null;
	}

	/**
	 * @param  null|string  $subParam
	 *
	 * @return bool
	 */
	protected function hasData($subParam = null)
	{
		return null !== $this->getData($subParam);
	}

	/**
	 * @var null | self
	 */
	private static $instance = null;

	public static function get_instance()
	{
		if (null === self::$instance) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	private function __clone()
	{
	}
}
